import logging
import os
import shutil
from contextlib import contextmanager
from datetime import datetime
from os.path import basename, join
from tempfile import mkdtemp

from ..jpssrdr import atms_sci_to_l0, cris_sci_to_l0, spacecraft_to_l0, viirs_sci_to_l0
from ..merge import merge
from ..rdrgen import build_rdr, filter_before, filter_group_orphans, iter_pkts
from ..stream import jpss_packet_stream
from .util import configure_logging

LOG = logging.getLogger("edosl0util.cli.rdrmerge")


# py2 compat
@contextmanager
def TemporaryDirectory(**kwds):
    dirpath = mkdtemp(**kwds)
    try:
        yield dirpath
    finally:
        if not os.environ.get("DEBUG", False):
            shutil.rmtree(dirpath)


def rdrs_to_process(inputs):
    # filter and validiate rdrs
    to_process = {
        (s, p): []
        for s in ("snpp", "j01")
        for p in ("RNSCA-RVIRS", "RCRIS-RNSCA", "RATMS-RNSCA")
    }
    for rdr in inputs:
        parts = basename(rdr).split("_")
        if len(parts) < 2:
            continue
        product = parts[0]
        # validate product
        if product not in ("RNSCA-RVIRS", "RCRIS-RNSCA", "RATMS-RNSCA"):
            LOG.debug("skipping unsupported product %s", rdr)
            continue
        # validate satellite
        sat = parts[1]
        sat = {"jpss1": "j01", "noaa20": "j01", "npp": "snpp"}.get(sat, sat)
        sat = "j01" if sat in ("jpss1", "noaa20") else sat
        if sat not in ("snpp", "j01"):
            LOG.debug("skipping unsupported satellite %s", sat)
            continue

        to_process[sat, product].append(rdr)
    return {(s, p): v for (s, p), v in to_process.items() if v}


sci_extractors = {
    "RNSCA-RVIRS": viirs_sci_to_l0,
    "RCRIS-RNSCA": cris_sci_to_l0,
    "RATMS-RNSCA": atms_sci_to_l0,
}


def extract_packets(sat, product, fpath, workdir="."):
    # use same start/end for all extracting since we don't care about file
    # timestamps
    start = datetime(1970, 1, 1)
    end = datetime.utcnow()

    fname = basename(fpath)
    pds = sci_extractors[product](sat, [fpath], start, end, workdir)
    outputs = [join(workdir, fname + ".sci.pds")]
    shutil.move(pds, outputs[0])

    scs = spacecraft_to_l0(sat, [fpath], start, end, workdir)
    for idx, pds in enumerate(scs):
        outputs.append(join(workdir, fname + ".sc%d.pds" % idx))
        shutil.move(pds, outputs[-1])

    return outputs


start_of_mission = {"snpp": datetime(2011, 10, 28), "noaa20": datetime(2017, 11, 18)}


def merge_rdrs(inputs):
    to_process = rdrs_to_process(inputs)

    # do each set of files separately to handle case where different products
    # are provided
    outputs = []
    for (sat, product), rdrs in to_process.items():
        LOG.info("handling %d files for %s %s", len(rdrs), sat, product)
        with TemporaryDirectory(dir=os.getcwd()) as tmpdir:
            # extract RDRs separately, name them after RDR
            pds = []
            for fpath in rdrs:
                LOG.debug("extracting %s", fpath)
                pds += extract_packets(sat, product, fpath, tmpdir)

            merged = join(tmpdir, basename(fpath) + ".merge.pds")
            LOG.debug("merging packet files to %s", merged)
            with open(merged, "wb") as fp:
                merge([jpss_packet_stream(open(p, "rb")) for p in pds], fp)

            # have to pre-filter "orphans" to prevent OrphanPacketError
            packets = filter_group_orphans(iter_pkts([merged]))
            packets = filter_before(packets, before=start_of_mission[sat])
            rdrs = build_rdr(
                sat, packets, aggr_type="full", output_dir=tmpdir, strict=False
            )
            assert len(rdrs) == 1, "Should have gotten a single RDR"
            rdr = rdrs[0]

            # copy back to CWD
            os.rename(join(tmpdir, rdr), rdr)
            outputs.append(rdr)

    return outputs


def main():
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument("-v", "--verbose", action="store_true")
    parser.add_argument("rdr", nargs="+", help="Any science rdr (RVIRS, RCRIS, RATMS)")
    args = parser.parse_args()
    configure_logging(args)

    if not args.rdr:
        parser.exit(1, "no RDR's provided")

    for o in merge_rdrs(args.rdr):
        LOG.info("created %s", o)


if __name__ == "__main__":
    main()
